@@ -971,18 +971,8 @@ soup_message_io_run_until_finish (SoupMessage   *msg,
 				  GCancellable  *cancellable,
 				  GError       **error)
 {
-	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
-	SoupMessageIOData *io = priv->io_data;
-
 	g_object_ref (msg);
 
-	if (io) {
-		g_return_if_fail (io->mode == SOUP_MESSAGE_IO_CLIENT);
-
-		if (io->read_state < SOUP_MESSAGE_IO_STATE_BODY_DONE)
-			io->read_state = SOUP_MESSAGE_IO_STATE_FINISHING;
-	}
-
 	if (!io_run_until (msg,
 			   SOUP_MESSAGE_IO_STATE_DONE,
 			   SOUP_MESSAGE_IO_STATE_DONE,
@@ -1159,8 +1149,7 @@ io_unpause_internal (gpointer msg)
 
 	g_return_val_if_fail (io != NULL, FALSE);
 
-	g_source_unref (io->unpause_source);
-	io->unpause_source = NULL;
+	g_clear_pointer (&io->unpause_source, g_source_unref);
 	io->paused = FALSE;
 
 	if (io->io_source)
